#include <SM130.h>

SM130 rfid;

int antennaStatus = 0x01;
int outputPortStatus = 0x00;

void setup(){
  Serial.begin(9600);
  rfid.begin(&Serial3);
  
  delay(500);
  
  Serial.println("!! initialization complete");
  Serial.println("-> now running program...");
  Serial.println();
  
  showCommandList();
  
  delay(500);
};

void loop(){
  //process command if received from serial interface
  if(Serial.available())
    int res = processCommand();
};

////////////////////////////////////
/// command process              ///
///       optional serial debug  ///
////////////////////////////////////

int processCommand(){
  switch (Serial.read()){
    case 'a':
    if(antennaStatus){
      antennaStatus = 0;
      rfid.switchAntenna(0);
      Serial.println("Antenna is OFF");
    }else{
      antennaStatus = 1;
      rfid.switchAntenna(1);
      Serial.println("Antenna is ON");
    }
    Serial.println();
    return 1;
    break;
    case 's':
    rfid.selectTAG();
    tagInfo();
    return 1;
    break;
    case 'w':
    writeTest();
    return 1;
    break;
    case 'd':
    readTest();
    return 1;
    break;
    case 'l':
    showCommandList();
    return 1;
    break;
    case 'k':
    rfid.seekForTAG();
    rfid.getData();
    return 1;
    break;
    case 'e':
    rfid.enterSleep();
    return 1;
    break;
    case 'h':
    rfid.haltTAG();
    return 1;
    break;
    case 'r':
    rfid.reset();
    return 1;
    break; 
    case 'i':
    Serial.print("Input port status: ");
    Serial.println(rfid.readPort(), HEX);
    return 1;
    break;
    case 'o':
    if(outputPortStatus == 0x00){
      outputPortStatus = rfid.writePort(0x03);
    }else{
      outputPortStatus = rfid.writePort(0x00);
    }
    return 1;
    break;
    default:
    Serial.println("Received unknown command");
    return 0;
    break;
  }
}

////////////////////////////////////
/// show command list            ///
////////////////////////////////////
void showCommandList(){
  Serial.println("Command list:");
  Serial.println("l -> show command list");
  Serial.println("r -> SM130 reset");
  Serial.println("a -> switch antenna ON/OFF");
  Serial.println("e -> enter sleep mode");
  Serial.println("     N.B. then, only hardware reset can wake up the module");
  Serial.println("o -> write to output port");
  Serial.println("i -> read from input port");
  Serial.println("s -> select tag");
  Serial.println("h -> halt TAG");
  Serial.println("k -> seek for TAG");
  Serial.println("w -> write block");
  Serial.println("d -> read block");
  Serial.println(); 
}

////////////////////////////////////
/// write test                  ///
////////////////////////////////////
void writeTest(){
  byte str[16];
  int i = 0;
  int blockNum = 0;
  
  Serial.println("-> Write routine started...");
  Serial.print("Enter max 16 ASCII characters (extra characters will be ignored): ");
  
  while(!Serial.available());
  
  while(Serial.available() && i < 16){
    str[i] = Serial.read();
    Serial.write(str[i]);
    Serial.print(" ");
    i++;
    delay(5);
  }
  
  delay(5);
  while(Serial.available()){
    Serial.read();
    delay(5);
  }
  
  for(i;i<16;i++)
    str[i] = 0;  
  Serial.println();
  
  blockNum = getNum();
  if(rfid.isTrailerBlock(blockNum)){
    Serial.println("You have inserted a trailer block number");
    Serial.println();
    return;
  }
  
  Serial.println();
  
  rfid.haltTAG();
  rfid.selectTAG();
  rfid.authenticate(blockNum);
  rfid.writeBlock(blockNum, str);
}

////////////////////////////////////
/// read test                    ///
////////////////////////////////////
void readTest(){
  int blockNum = 0;
  int i = 0;
  
  Serial.println("-> Read routine started...");
  
  blockNum = getNum();

  if(rfid.haltTAG()){
    if(rfid.selectTAG()){
      if(rfid.authenticate(blockNum)){
        int i = rfid.readBlock(blockNum);
        if(i>0){
          Serial.print("Data (HEX): ");
          for(int i=0;i<BLOCK_SIZE;i++){
            Serial.print(rfid.dataBuffer[5+i], HEX);
            Serial.print(" ");
          }
          Serial.println();
          Serial.print("Data ASCII: ");
          for(int i=0;i<BLOCK_SIZE;i++){
            Serial.write(rfid.dataBuffer[5+i]);
            Serial.print(" ");
          }
          Serial.println();
          Serial.println();
        }else{
          Serial.print("Error on reading TAG: ");
          Serial.println(i);
        }
      }
    }
  }
}


////////////////////////////////////
/// print TAG informations       ///
////////////////////////////////////

void tagInfo(){
  Serial.println("-> TAG informations");
  Serial.print("TAG type: ");
  Serial.print(rfid.getTAGType());
  Serial.println();

  Serial.print("TAG serial number: ");
  switch(rfid.currentTAGType){
    case mifare1k:
    case mifare4k:
    for(int i=0;i<4;i++){
      Serial.print(rfid.currentTAGSerial[i], HEX);
      Serial.print(" ");
    }
    break;
    case mifareUltralight:
    for(int i=0;i<4;i++){
      Serial.print(rfid.currentTAGSerial[i], HEX);
      Serial.print(" ");
    }
    break;
    default:
    Serial.println("Unknown");
    break;
  }
  Serial.println();
}

////////////////////////////////////////
///     serial to integer            ///
////////////////////////////////////////

int getNum(){
  int blockNum = 0;
  int i = 0;
  char temp[3];
  temp[2] = '\0';
  
  Serial.print("Enter block number: ");
  
  while(!Serial.available());
  
  i = 0;
  while(Serial.available() && i<2){
    temp[i] = Serial.read();
    i++;
    delay(5);
  }
  if(i==1){
    temp[1] = temp[0];
    temp[0] = '0';
    blockNum = atoi(temp);
  }else{
    blockNum = atoi(temp);
  }
  Serial.println(blockNum); 
  
  delay(5);
  while(Serial.available()){
    Serial.read();
    delay(5);
  }
  
  return blockNum;
}
